#!/usr/bin/env python
"""
This is a setup.py script generated by py2applet

Usage:
    python setup.py py2app
"""

from setuptools import setup
import shutil
import os
import stat
import subprocess

# Trick to make this run properly
import sys
sys.argv += ['py2app']

subprocess.call(['make','fab'], cwd='../..')

# Delete old build stuff
try:    shutil.rmtree('build')
except OSError: pass

try:    shutil.rmtree('koko')
except OSError: pass

try:    os.remove('kokopelli.py')
except OSError: pass

# This is the pythons script that we're bundling into an application.
shutil.copy('../../bin/kokopelli','kokopelli.py')
shutil.copytree('../../bin/koko','koko')

APP = ['kokopelli.py']
DATA_FILES = ['../../bin/math_png',
              '../../bin/math_stl',
              '../../bin/math_svg',
              '../../bin/math_dot',
              '../../bin/math_stats',
              '../../bin/koko/lib/shapes.py',
              '../../bin/koko/lib/text.py',
              '../../bin/koko/c/shapes2d.py',
              '../../bin/koko/c/shapes3d.py',
              'cba_icon.png']
OPTIONS = {'argv_emulation': True,
           'iconfile':'cba.icns'}

# Run py2app to bundle everything.
setup(
    app=APP,
    data_files=DATA_FILES,
    options={'py2app': OPTIONS},
    setup_requires=['py2app'],
)

# Copy the boost threads library.
for lib in ['thread','system','chrono']:
    shutil.copy('/usr/local/lib/libboost_%s-mt.dylib' % lib,
                'dist/kokopelli.app/Contents/Frameworks/libboost_%s-mt.dylib' % lib)

# Copy libtree
os.mkdir('dist/kokopelli.app/Contents/lib')
shutil.copy('../../lib/libtree.dylib', 
            'dist/kokopelli.app/Contents/lib/libtree.dylib')
            
executable = stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH | \
             stat.S_IRUSR | stat.S_IWUSR

for m in ['math_png','math_stl','math_svg','math_dot','math_stats']:
    # Change executable linking so that math tools link against
    # the local copy of boost threads and system
    os.system('install_name_tool -change ' +
              '"/usr/local/lib/libboost_thread-mt.dylib" ' +
              '"@executable_path/../Frameworks/libboost_thread-mt.dylib" ' +
              'dist/kokopelli.app/Contents/Resources/%s' % m)
    os.system('install_name_tool -change ' +
              '"/usr/local/lib/libboost_system-mt.dylib" ' +
              '"@executable_path/../Frameworks/libboost_system-mt.dylib" ' +
              'dist/kokopelli.app/Contents/Resources/%s' % m)
    
    # Make executable
    os.chmod('dist/kokopelli.app/Contents/Resources/%s' % m, executable)

editable = stat.S_IWUSR | stat.S_IRUSR
for lib in ['chrono', 'thread', 'system']:
    os.chmod('dist/kokopelli.app/Contents/Frameworks/libboost_%s-mt.dylib' % lib,
             editable)
             
# Relink boost libraries so that they talk to each other
os.system('install_name_tool -change ' +
          '"/usr/local/lib/libboost_chrono-mt.dylib" ' +
          '"@executable_path/../Frameworks/libboost_chrono-mt.dylib" ' +
          'dist/kokopelli.app/Contents/Frameworks/libboost_thread-mt.dylib')
os.system('install_name_tool -change ' +
          '"/usr/local/lib/libboost_system-mt.dylib" ' +
          '"@executable_path/../Frameworks/libboost_system-mt.dylib" ' +
          'dist/kokopelli.app/Contents/Frameworks/libboost_thread-mt.dylib')
os.system('install_name_tool -change ' +
          '"/usr/local/lib/libboost_system-mt.dylib" ' +
          '"@executable_path/../Frameworks/libboost_system-mt.dylib" ' +
          'dist/kokopelli.app/Contents/Frameworks/libboost_chrono-mt.dylib')


# Copy the readme and examples into the distribution directory, then zip it up
shutil.copy('README','dist')
shutil.copy('../../../../Web/examples.zip', 'dist/examples.zip')
os.system('cd dist; unzip examples.zip; rm -rf __MACOSX;'+
          'zip -r kokopelli.zip kokopelli.app README examples')
shutil.rmtree('build')
shutil.rmtree('koko')
shutil.os.remove('kokopelli.py')