import ctypes
import os
import sys
import threading

filename = os.path.abspath(sys.argv[0])
filename = '/'.join(filename.split('/')[:-2])+'/lib/libtree'
filename += '.dylib' if 'Darwin' in os.uname() else '.so'
    
try:
    libtree = ctypes.CDLL(filename)
except OSError:
    print 'Warning: libtree not found.  c solver disabled!'
    libtree = None
else:
    # Helper functions for pointer and pointer to pointer
    def p(t):
        return ctypes.POINTER(t)
    def pp(t):
        return p(p(t))
    
    ################################################################################
    
    # util/region.h
    from region import Region
    
    libtree.split.argtypes = [Region, p(Region), ctypes.c_int]
    libtree.split.restype  = ctypes.c_int
    
    libtree.split_xy.argtypes = [Region, p(Region), ctypes.c_int]
    libtree.split_xy.restype  = ctypes.c_int
    
    libtree.octsect.argtypes = [Region, Region*8]
    libtree.octsect.restype  = ctypes.c_uint8

    libtree.octsect_overlap.argtypes = [Region, Region*8]
    libtree.octsect_overlap.restype  = ctypes.c_uint8
    
    libtree.build_arrays.argtypes = [p(Region)] + [ctypes.c_float]*6
    libtree.free_arrays.argtypes  = [p(Region)]
    
    # util/vec3f.h
    from vec3f import Vec3f
    libtree.deproject.argtypes = [Vec3f, ctypes.c_float*4]
    libtree.deproject.restype  =  Vec3f

    libtree.project.argtypes = [Vec3f, ctypes.c_float*4]
    libtree.project.restype  =  Vec3f
    
    # util/png.c
    libtree.save_png16L.argtypes = [p(ctypes.c_char), ctypes.c_int,
                                    ctypes.c_int, ctypes.c_float*6,
                                    pp(ctypes.c_uint16)]
    libtree.downsample_image.argtypes = [ctypes.c_int, ctypes.c_int,
                                         pp(ctypes.c_uint16),
                                         pp(ctypes.c_uint8)]
    libtree.triple_image.argtypes = [ctypes.c_int, ctypes.c_int,
                                     pp(ctypes.c_uint8), pp(ctypes.c_uint8*3)]
                                     
    libtree.count_by_color.argtypes = [p(ctypes.c_char), ctypes.c_int,
                                       ctypes.c_int, ctypes.c_uint32,
                                       p(ctypes.c_uint32)]
    
    ################################################################################
        
    class MathTreeP(ctypes.c_void_p):   pass
    class PackedTreeP(ctypes.c_void_p): pass
    
    # tree/solver.h
    libtree.render8.argtypes = [PackedTreeP, Region,
                                pp(ctypes.c_uint8), p(ctypes.c_int)]
    
    # tree/tree.h
    
    libtree.free_tree.argtypes = [MathTreeP]
    
    libtree.print_tree.argtypes = [MathTreeP]
    libtree.fprint_tree.argtypes = [MathTreeP, ctypes.c_int]
    libtree.fprint_tree_verbose.argtypes = [MathTreeP, ctypes.c_int]
    
    libtree.make_packed.argtypes = [MathTreeP]
    libtree.make_packed.restype  =  PackedTreeP

    libtree.free_packed.argtypes = [PackedTreeP]
    
    # Binary numerical operations
    for f in ['add','sub','div','mul', 'min', 'max', 'pow']:
        f += '_tree'
        getattr(libtree, f).argtypes = [MathTreeP, MathTreeP]
        getattr(libtree, f).restype  =  MathTreeP
        
    # Unary operations
    #for f in ['square_n', 'sqrt_n', 'sin_n', 'cos_n', 'tan_n', 'neg_n', 'buf_n']:
    for f in ['square', 'sqrt', 'neg']:
        f += '_tree'
        getattr(libtree, f).argtypes = [MathTreeP]
        getattr(libtree, f).restype  =  MathTreeP
        
    # Nonary operations
    for f in ['X', 'Y', 'Z']:
        f += '_tree'
        getattr(libtree, f).argtypes = []
        getattr(libtree, f).restype  = MathTreeP
        
    # Special case for constant_n
    libtree.constant_tree.argtypes = [ctypes.c_float]
    libtree.constant_tree.restype  = MathTreeP
    
    libtree.map_tree.argtypes = [MathTreeP]*4
    libtree.map_tree.restype  =  MathTreeP
    
    libtree.clone_tree.argtypes = [MathTreeP]
    libtree.clone_tree.restype  =  MathTreeP
    
    ################################################################################
                           
    # asdf/asdf.h
    from asdf import _ASDF
    
    libtree.build_asdf.argtypes = [PackedTreeP, Region, p(ctypes.c_int)]
    libtree.build_asdf.restype  =  p(_ASDF)

    libtree.asdf_root.argtypes = [PackedTreeP, Region]
    libtree.asdf_root.restype  =  p(_ASDF)
    
    libtree.asdf_scale.argtypes = [p(_ASDF), ctypes.c_float]
    
    libtree.find_dimensions.argtypes = [p(_ASDF)] + [p(ctypes.c_int)]*3
    libtree.get_d_from_children.argtypes = [p(_ASDF)]
    
    libtree.free_asdf.argtypes = [p(_ASDF)]
    
    libtree.interpolate.argtypes = [p(ctypes.c_float)] + [ctypes.c_float]*9
    libtree.interpolate.restype  = ctypes.c_float
    
    libtree.get_depth.argtypes = [p(_ASDF)]
    libtree.get_depth.restype  = ctypes.c_int
    
    libtree.make_extrapolation.argtypes = [p(_ASDF)]
    
    libtree.set_depth.argtypes = [p(_ASDF), ctypes.c_int]
    libtree.clear_depth.argtypes = [p(_ASDF)]
    
    # asdf/import.h
    libtree.import_vol_resampled.argtypes = [
                                p(ctypes.c_char), ctypes.c_int,
                                ctypes.c_int, ctypes.c_int, ctypes.c_int,
                                ctypes.c_float, ctypes.c_float]
    libtree.import_vol_resampled.restype  = p(_ASDF)
    
    libtree.import_vol_region.argtypes = [p(ctypes.c_char)] +\
                                         [ctypes.c_int]*3 + \
                                         [Region, ctypes.c_int, ctypes.c_float]
    libtree.import_vol_region.restype  = p(_ASDF)
    
    libtree.import_vol.argtypes = [p(ctypes.c_char), ctypes.c_int,
                                   ctypes.c_int, ctypes.c_int,
                                   ctypes.c_float, ctypes.c_float]
    libtree.import_vol.restype  = p(_ASDF)
    
    # asdf/render.h
    libtree.render_asdf.argtypes = [p(_ASDF), Region, ctypes.c_float*4,
                                    pp(ctypes.c_uint16), pp(ctypes.c_uint16),
                                    pp(ctypes.c_uint8*3)]
    libtree.draw_asdf_cells.argtypes = [p(_ASDF), Region, pp(ctypes.c_uint8*3)]
    
    # asdf/file_io.h
    libtree.asdf_write.argtypes = [p(_ASDF), p(ctypes.c_char)]
                                   
    libtree.asdf_read.argtypes = [p(ctypes.c_char)]
    libtree.asdf_read.restype  =  p(_ASDF)
    
    # asdf/mesh.h
    libtree.count_leafs.argtypes = [p(_ASDF)]
    libtree.count_leafs.restype  = ctypes.c_int

    libtree.count_tris.argtypes = [p(_ASDF)]
    libtree.count_tris.restype  = ctypes.c_int
    
    libtree.triangulate.argtypes = [p(_ASDF),
                                    p(ctypes.c_float),  p(ctypes.c_uint32),
                                    p(ctypes.c_uint32), p(ctypes.c_uint32),
                                    p(ctypes.c_int)]
    
    libtree.find_bounds.argtypes = [p(ctypes.c_float), ctypes.c_int,
                                    p(Vec3f), p(Vec3f)]
    
    libtree.find_radius.argtypes = [p(ctypes.c_float), ctypes.c_int, Vec3f]
    libtree.find_radius.restype  = ctypes.c_float

    libtree.increase_indices.argtypes = [p(ctypes.c_uint32), ctypes.c_uint32,
                                         ctypes.c_uint32]
    
    libtree.write_stl.argtypes = [p(ctypes.c_uint32), ctypes.c_uint32,
                                  p(ctypes.c_float), p(ctypes.c_char)]

    from interval import Interval
    
    libtree.write_mesh.argtypes = [p(ctypes.c_char)] + [Interval]*3 + [
                                   p(ctypes.c_uint32), ctypes.c_uint32,
                                   p(ctypes.c_float), ctypes.c_uint32]

    libtree.read_mesh_header.argtypes = [p(ctypes.c_char)] + [p(Interval)]*3 +\
                                        [p(ctypes.c_uint32)]*2

    libtree.read_mesh.argtypes = [p(ctypes.c_char),
                                  p(ctypes.c_uint32), ctypes.c_uint32,
                                  p(ctypes.c_float),  ctypes.c_uint32]
    del p, pp