import ctypes
from math import sin, cos, radians

from libtree import libtree

class Vec3f(ctypes.Structure):
    _fields_ = [('x', ctypes.c_float),
                ('y', ctypes.c_float),
                ('z', ctypes.c_float)]
    def __str__(self):
        return "(%g, %g, %g)" % (self.x, self.y, self.z)
    def __repr__(self):
        return "Vec3f(%g, %g, %g)" % (self.x, self.y, self.z)
    def __add__(self, rhs):
        return Vec3f(self.x + rhs.x, self.y + rhs.y, self.z + rhs.z)
    def __div__(self, rhs):
        return Vec3f(self.x/rhs, self.y/rhs, self.z/rhs)
    def project(self, alpha, beta):
        ''' Cell frame to view frame. '''
        M = (ctypes.c_float*4)(cos(radians(alpha)), sin(radians(alpha)),
                               cos(radians(beta)),  sin(radians(beta)))
        return libtree.project(self, M)
        
    def deproject(self, alpha, beta):
        ''' View frame to cell frame. '''
        M = (ctypes.c_float*4)(cos(radians(alpha)), sin(radians(alpha)),
                               cos(radians(beta)),  sin(radians(beta)))
        return libtree.deproject(self, M)